<?php
/*
 * _form: Drupal\user_import\Form\UserImportForm
 * */
namespace Drupal\user_import\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user_import\Service\MysqlConnectService;
use Drupal\user_import\Service\AddUserService;
use Symfony\Component\DependencyInjection\ContainerInterface;

class UserImportForm extends FormBase{
    protected $mysqlConnectService;
    protected $addUserService;

    public function __construct(MysqlConnectService $mysqlConnectService, AddUserService $addUserService) {
        $this->mysqlConnectService = $mysqlConnectService;
        $this->addUserService = $addUserService;
    }

    public static function create(ContainerInterface $container) {
        $mysqlConnect = $container -> get('user_import.mysql_connect');
        $addUser = $container -> get('user_import.add_user');
        return new static ($mysqlConnect, $addUser);
    }

    public function getFormId() {
        return 'user_import_form';
    }

    public function buildForm(array $form, FormStateInterface $form_state) {
        $ldapAccounts = array('apple', 'banana', 'lemon', 'orange');
        $existingAccounts = array('banana', 'lemon');

        $header = [
            'check' => $this->t(''),
            'present' => $this->t('Exists'),
            'username' => $this->t('Username'),
            'userdata' => $this->t('User data'),
        ];

        $form['table'] = array(
            '#type' => 'table',
            '#title' => 'LDAP accounts',
            //'#header' => $header,
            '#tree' => TRUE,
        );

        for ($i=0; $i<sizeof($ldapAccounts); $i++) {
            $form['table'][$i]["check"] = array(
                '#type' => 'checkboxes',
                '#title' => $ldapAccounts[$i],
                '#options' =>array($ldapAccounts[$i] => ''),
                '#title_display' => 'invisible',
            );
            $markup = (in_array($ldapAccounts[$i], $existingAccounts)? '<div class="mark"></div>':'<div></div>');
            $form['table'][$i]['present'] = array(
                '#type' => 'markup',
                '#markup' => $markup,
            );
            $form['table'][$i]['username'] = array(
                '#type' => 'label',
                '#title' => $ldapAccounts[$i],
            );
            $form['table'][$i]['userdata'] = array(
                '#type' => 'details',
                '#title' => $this -> t('details'),
            );
            $form['table'][$i]['userdata']['details'] = array(
                '#type' => 'markup',
                '#markup' => '<p>data</p>',
            );

            //submit button
            $form['add'] = array(
                '#type' => 'submit',
                '#value' => $this->t('Add/Update user from LDAP'),
            );
        }
        return $form;
    }



    public function validateForm(array &$form, FormStateInterface $form_state) {
    }

    private function implode_recur($separator, $arrayvar) {
        $output = "";
        foreach ($arrayvar as $av)
            if (is_array ($av))
                $output .= $this->implode_recur($separator, $av); // Recursive array
            else
                $output .= $separator.$av;

        return $output;
        }

    public function submitForm(array &$form, FormStateInterface $form_state) {
        //$this -> mysqlConnectService -> connect()

        foreach ($form_state->getValues() as $key => $value) {

            if (is_array($value)){
                drupal_set_message($key.' '.$this->implode_recur(" ", $value));
                //drupal_set_message($key.' '.implode(" ", $value));
            } else {
                drupal_set_message($key . ': ' . $value);
            }
        }
    }
}